import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_baidu_mapapi_base/flutter_baidu_mapapi_base.dart';
import 'package:flutter_baidu_mapapi_map/flutter_baidu_mapapi_map.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';
import 'package:flutter_yingyan_trace_demo/general/utils.dart';
import 'package:flutter_yingyan_trace_demo/history_track/track_analysis_bar.dart';
import '../customWidgets/setting_button.dart';
import 'history_track_setting_page.dart';

/// 历史轨迹查询
class HistoryTrackPage extends StatefulWidget {
  @override
  _HistoryTrackPageState createState() => _HistoryTrackPageState();
}

class _HistoryTrackPageState extends State<HistoryTrackPage>
    with TickerProviderStateMixin {
  BMFMapController _mapController;
  BMFPolyline _polyline;
  bool _distanceWidgetVisiable = true;
  bool _traceAnalysisButtonSelected = false;

  QueryHistoryTrackResult _historyTrackResult;
  QueryHistoryTrackOption _historyTrackOption;

  /// 驾驶行为选中状态
  List<bool> _drivingBehaviourStatus = [false, false, false, false];

  /// 驾驶行为marker
  Map<int, List<BMFMarker>> _drivingBehaviourMarkerMap = {
    0: [],
    1: [],
    2: [],
    3: []
  };

  /// 停留点marker
  List<BMFMarker> _stayPointMarkers = [];

  /// 里程信息动画
  AnimationController _animationController;
  Animation _distanceWidgetShowAnimation;

  /// 轨迹分析动画
  AnimationController _traceAnalysisBarAnimationController;
  Animation _traceAnalysisBarShowAnimation;

  @override
  void initState() {
    super.initState();

    ///
    _animationController =
        AnimationController(duration: Duration(milliseconds: 500), vsync: this);
    _distanceWidgetShowAnimation = Tween(
      begin: Offset(0, 1),
      end: Offset(0, 0),
    ).animate(_animationController);

    ///
    _traceAnalysisBarAnimationController =
        AnimationController(duration: Duration(milliseconds: 500), vsync: this);
    _traceAnalysisBarShowAnimation = Tween(
      begin: Offset(0, 1),
      end: Offset(0, 0),
    ).animate(_traceAnalysisBarAnimationController);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: BMFAppBar(
        title: "轨迹查询",
        actions: [
          SettingButton(
            onTap: _onTapSetting,
          )
        ],
      ),
      body: Stack(
        children: [
          _createMap(),
          _distanceWidget(),
          _traceAnalysisButton(),
          _traceAnalysisBar(),
        ],
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    TraceController.shareInstance?.release();

    _animationController.dispose();
  }

  /// 设置 tap
  void _onTapSetting() {
    _distanceWidgetVisiable = false;
    _hideDistanceInfoWithAnimation();

    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => HistoryTrackSettingPage(
                  callback: _settingPageCallback,
                )));
  }

  /// 设置页回调
  void _settingPageCallback(QueryHistoryTrackOption option) async {
    _historyTrackOption = option;
    bool flag = await TraceController.shareInstance.queryHistoryTrack(
      queryHistoryTrackOption: option,
      trackCallBack: TrackCallBack(
          onQueryHistoryTrackCallBack: (QueryHistoryTrackResult result) async {
        print('-- 查询某终端实体在一段时间内的轨迹回调 result = ${result?.toMap()}');

        showToast(msg: result.message);

        if (result.status != 0) {
          return;
        }

        /// 移除marker 和 polyline
        await _mapController.cleanAllMarkers();

        if (_polyline != null) {
          _mapController.removeOverlay(_polyline.Id);
        }

        /// 起点marker
        if (result.startPoint != null) {
          BMFCoordinate position = BMFCoordinate(
              result.startPoint.location.latitude,
              result.startPoint.location.longitude);
          BMFMarker marker = BMFMarker(
              position: position, icon: "images/general/icon_start.png");
          await _mapController.addMarker(marker);
        }

        /// 终点marker
        if (result.endPoint != null) {
          BMFCoordinate position = BMFCoordinate(
              result.endPoint.location.latitude,
              result.endPoint.location.longitude);
          BMFMarker marker = BMFMarker(
              position: position, icon: "images/general/icon_end.png");
          await _mapController.addMarker(marker);
        }

        /// 轨迹polyline
        if (result.trackPoints != null && result.trackPoints.isNotEmpty) {
          List<BMFCoordinate> coords = [];
          result.trackPoints.forEach((element) {
            BMFCoordinate coord = BMFCoordinate(
                element.location.latitude, element.location.longitude);
            coords.add(coord);
          });

          BMFPolyline polyline = BMFPolyline(
              coordinates: coords, indexs: [], colors: [Colors.blue]);
          await _mapController.addPolyline(polyline);
          _polyline = polyline;

          BMFCoordinateBounds bounds = getVisibleMapRect(coords);
          _mapController.setVisibleMapRectWithPaddingHandler(
              visibleMapBounds: bounds,
              insets:
                  EdgeInsets.only(left: 15, top: 15, right: 15, bottom: 100),
              animated: true);
        }

        setState(() {
          _historyTrackResult = result;
          _distanceWidgetVisiable = true;
        });
        _hideTraceAnalysisBar();
        _showDistanceInfoWithAnimation();
      }),
    );
    print('-- 查询某终端实体在一段时间内的轨迹 flag = $flag');
  }

  /// 动画
  void _showDistanceInfoWithAnimation() {
    _animationController.forward();
  }

  void _hideDistanceInfoWithAnimation() {
    _animationController.reverse();
  }

  void _showTraceAnalysisBarWithAnimation() {
    _traceAnalysisBarAnimationController.forward();
  }

  void _hideTraceAnalysisBarWithAnimation() {
    _traceAnalysisBarAnimationController.reverse();
  }

  ///
  void _onTapTraceAnalysisButton() {
    _traceAnalysisButtonSelected = !_traceAnalysisButtonSelected;
    _traceAnalysisButtonSelected
        ? _showTraceAnalysisBarWithAnimation()
        : _hideTraceAnalysisBarWithAnimation();
  }

  /// 轨迹分析回调
  void _traceAnalysisBarCallback(int idx, bool selected) {
    if (idx != 4) {
      _drivingBehaviourStatus[idx] = selected;

      if (selected) {
        _drivingBehaviourAnalysis();
      } else {
        List markers = _drivingBehaviourMarkerMap[idx];
        _mapController.removeMarkers(markers);
        _drivingBehaviourMarkerMap[idx].clear();
      }
    } else {
      if (selected) {
        _stayPointAnalysis();
      } else {
        _mapController.removeMarkers(_stayPointMarkers);
        _stayPointMarkers.clear();
      }
    }
  }

  /// 驾驶行为分析
  void _drivingBehaviourAnalysis() async {
    if (_historyTrackOption == null) return;

    DrivingBehaviourAnalysisOption drivingBehaviourAnalysisOption =
        DrivingBehaviourAnalysisOption(
            tag: 1,
            serviceId: serviceID,
            entityName: entityName,
            startTime: _historyTrackOption.startTime,
            endTime: _historyTrackOption.endTime,
            speedingThreshold: 2,
            thresholdOption:
                DrivingBehaviorThresholdOption(speedingThreshold: 0),
            processOption: QueryTrackProcessOption(
                denoise: true,
                vacuate: true,
                mapMatch: true,
                radiusThreshold: 20,
                transportMode: TrackProcessOptionTransportMode.AUTO),
            outputCoordType: CoordType.BD09LL);

    bool flag = await TraceController.shareInstance.analyzeDrivingBehaviour(
        drivingBehaviourAnalysisOption: drivingBehaviourAnalysisOption,
        analysisCallBack: AnalysisCallBack(onDrivingBehaviourAnalysisCallBack:
            (DrivingBehaviourAnalysisResult result) {
          print('-- 驾驶行为分析回调 result = ${result?.toMap()}');
          showToast(msg: result.message);
          if (result.status != 0) return;

          _addDrivingBehaviourMarkers(result);
        }));

    print('-- 驾驶行为分析 flag = $flag');
  }

  /// 驾驶行为marker
  void _addDrivingBehaviourMarkers(DrivingBehaviourAnalysisResult result) {
    /// 超速
    if (_drivingBehaviourStatus[0]) {
      result.speedings?.forEach((element) {
        element.points.forEach((subElement) {
          BMFMarker marker = BMFMarker(
            position: BMFCoordinate(
              subElement.location.latitude,
              subElement.location.longitude,
            ),
            icon: "images/general/icon_gcoding.png",
          );
          _drivingBehaviourMarkerMap[0].clear();
          _drivingBehaviourMarkerMap[0].add(marker);
        });
      });
      _mapController.addMarkers(_drivingBehaviourMarkerMap[0]);
    }

    /// 急刹车
    if (_drivingBehaviourStatus[1]) {
      result.harshBreakingPoints.forEach((element) {
        BMFMarker marker = BMFMarker(
          position: BMFCoordinate(
            element.location.latitude,
            element.location.longitude,
          ),
          icon: "images/general/icon_gcoding.png",
        );
        _drivingBehaviourMarkerMap[1].clear();
        _drivingBehaviourMarkerMap[1].add(marker);
      });
      _mapController.addMarkers(_drivingBehaviourMarkerMap[1]);
    }

    /// 急变速
    if (_drivingBehaviourStatus[2]) {
      result.harshAccelerationPoints.forEach((element) {
        BMFMarker marker = BMFMarker(
          position: BMFCoordinate(
            element.location.latitude,
            element.location.longitude,
          ),
          icon: "images/general/icon_gcoding.png",
        );
        _drivingBehaviourMarkerMap[2].clear();
        _drivingBehaviourMarkerMap[2].add(marker);
      });
      _mapController.addMarkers(_drivingBehaviourMarkerMap[2]);
    }

    /// 急转弯
    if (_drivingBehaviourStatus[3]) {
      result.harshSteeringPoints.forEach((element) {
        BMFMarker marker = BMFMarker(
          position: BMFCoordinate(
            element.location.latitude,
            element.location.longitude,
          ),
          icon: "images/general/icon_gcoding.png",
        );
        _drivingBehaviourMarkerMap[3].clear();
        _drivingBehaviourMarkerMap[3].add(marker);
      });
      _mapController.addMarkers(_drivingBehaviourMarkerMap[3]);
    }
  }

  /// 停留点分析
  void _stayPointAnalysis() async {
    if (_historyTrackOption == null) return;

    StayPointAnalysisOption stayPointAnalysisOption = StayPointAnalysisOption(
        tag: 1,
        serviceId: serviceID,
        entityName: entityName,
        startTime: _historyTrackOption.startTime,
        endTime: _historyTrackOption.endTime,
        stayRadius: 20,
        processOption: QueryTrackProcessOption(
            denoise: true,
            vacuate: true,
            mapMatch: true,
            radiusThreshold: 20,
            transportMode: TrackProcessOptionTransportMode.AUTO),
        outputCoordType: CoordType.BD09LL);

    bool flag = await TraceController.shareInstance.analyzeStayPoint(
        stayPointAnalysisOption: stayPointAnalysisOption,
        analysisCallBack: AnalysisCallBack(
            onStayPointAnalysisCallBack: (StayPointAnalysisResult result) {
          print('-- 停留点分析回调 result = ${result?.toMap()}');

          showToast(msg: result.message);
          if (result.status != 0) return;

          /// 停留点marker
          _mapController.removeMarkers(_stayPointMarkers);
          _stayPointMarkers.clear();

          if (result.stayPoints != null && result.stayPoints.isNotEmpty) {
            result.stayPoints.forEach((element) {
              BMFMarker marker = BMFMarker(
                position: BMFCoordinate(
                  element.location.latitude,
                  element.location.longitude,
                ),
                icon: "images/general/icon_gcoding.png",
              );
              _stayPointMarkers.add(marker);
            });
            _mapController.addMarkers(_stayPointMarkers);
          }
        }));

    print('-- 停留点分析 flag = $flag');
  }

  void _traceAnalysisBarCloseCallback() {
    _hideTraceAnalysisBar();
  }

  void _hideTraceAnalysisBar() {
    _traceAnalysisButtonSelected = false;
    _hideTraceAnalysisBarWithAnimation();
  }

  /// 轨迹分析
  Widget _traceAnalysisButton() {
    return Positioned(
      right: 0,
      bottom: 100,
      child: GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: Container(
          padding: EdgeInsets.all(10),
          child: Image.asset(
            "images/general/bg_btn_all_control.png",
            width: 35,
            height: 35,
          ),
        ),
        onTap: _onTapTraceAnalysisButton,
      ),
    );
  }

  /// 轨迹分析bar
  Widget _traceAnalysisBar() {
    return Positioned(
      left: 0,
      right: 0,
      bottom: 0,
      child: SafeArea(
        child: SlideTransition(
          position: _traceAnalysisBarShowAnimation,
          child: TrackAnalysisBar(
            callback: _traceAnalysisBarCallback,
            closeCallback: _traceAnalysisBarCloseCallback,
          ),
        ),
      ),
    );
  }

  /// 里程信息
  Widget _distanceWidget() {
    return Positioned(
      left: 10,
      right: 10,
      bottom: 0,
      child: SafeArea(
        child: Visibility(
          visible: _distanceWidgetVisiable,
          child: SlideTransition(
            position: _distanceWidgetShowAnimation,
            child: Container(
              color: Colors.white,
              padding: EdgeInsets.all(10),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text("总里程：${_historyTrackResult?.distance ?? 0} 米"),
                  Text("收费里程：${_historyTrackResult?.tollDistance ?? 0} 米"),
                  Text("低速里程：${_historyTrackResult?.lowSpeedDistance ?? 0} 米"),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  /// 地图
  BMFMapWidget _createMap() {
    BMFMapOptions options = BMFMapOptions(
      zoomLevel: 16,
      center: BMFCoordinate(40.056056, 116.308102),
    );
    return BMFMapWidget(
      mapOptions: options,
      onBMFMapCreated: (BMFMapController controller) {
        _mapController = controller;
      },
    );
  }
}
